#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.data

Module for handling data. Data can be accessed using the resource function.

"""

import os

def resource(*args):

    """ passes a file path for a data resource specified """

    return os.path.join(os.path.dirname(__file__), *args)



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.utils

Module providing utility functions for scikit-chem
"""

from .suppress import Suppressor



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.core.bond

Defining chemical bonds in scikit-chem.
"""

import rdkit.Chem
from skchem.core import Atom
from skchem.core import ChemicalObject

class Bond(rdkit.Chem.rdchem.Bond, ChemicalObject):

    """
    Class representing a chemical bond in scikit-chem.

    """

    @property
    def order(self):

        """
        The order of the bond.

        Parameters
        ----------

        None

        Returns
        -------

        bond :int

        """

        return self.GetBondTypeAsDouble()

    @order.setter
    def order(self, value):

        """ Set the order of the bond.  Not implemented."""

        raise NotImplementedError

    @property
    def atoms(self):

        """ Return an iterable of the atoms involved in the bond. """

        return [Atom.from_super(self.GetBeginAtom()), Atom.from_super(self.GetEndAtom())]

    @atoms.setter
    def atoms(self, value):

        """ Set the atoms of the molecule.  Not implemented. """

        raise NotImplementedError

    def draw(self):

        """ Draw the bond inline. """

        return '{}{}{}'.format(self.atoms[0].element, \
            '-' if self.order == 1 else self.GetSmarts(), \
            self.atoms[0].element)

    def to_dict(self):

        """ Convert to a dictionary representation. """

        return {"b": self.GetBeginAtomIdx(), "e": self.GetEndAtomIdx(), "o": self.order}

    def __repr__(self):
        return '<{klass} type="{bond}" at {address}>'.format(klass=self.__class__.__name__, \
            bond=self.draw(), \
            address=hex(id(self)))

    def __str__(self):
        return self.draw()



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

""" A cheminformatics library aiming to integrate into the Scientific Python Stack """

__version__ = '0.0.1dev'

from .core import Mol
from .io import read_sdf, read_smiles



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

""" Tests for scikit chemize """

from unittest import TestCase
from skchem.data import resource

from rdkit import Chem

from skchem.io import read_sdf
from skchem.core import Mol
from skchem.descriptors import skchemize
import pandas as pd

class TestSKChemize(TestCase):

    """ test class for skchemize """

    def test_modifies_rdkit_fps(self):

        '''Check to see if the fingerprint now gives a Series object'''

        m = Mol.from_smiles('C')
        func = skchemize(Chem.RDKFingerprint)
        self.assertTrue(isinstance(func(m), pd.Series))

    def test_fingerprint_is_same(self):

        '''Check if the fingerprint is the same'''

        m = Mol.from_smiles('c1ccccc1')
        f = skchemize(Chem.RDKFingerprint)
        gold = list(Chem.RDKFingerprint(m))
        self.assertTrue(f(m).tolist() == gold)

    def test_gives_dataframe(self):

        '''check to see if a dataframe is obtained'''

        f = skchemize(Chem.RDKFingerprint)
        df = read_sdf(resource('test_sdf', 'multi_molecule-simple.sdf'))
        self.assertTrue(isinstance(df.structure.apply(f), pd.DataFrame))



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.target_prediction.target_prediction

Target prediction base class
"""

import pandas as pd
from skchem import Mol

class AbstractTargetPredictionAlgorithm(object):

    """abstract target prediction class, inherit from this for a target prediction algorithm"""

    def predict(self, obj):

        """ Predict the targets for a compound or compound DataFrame """

        if isinstance(obj, Mol):
            return self._m_predict(obj)

        if isinstance(obj, pd.Series) or isinstance(obj, pd.DataFrame):
            return self._df_predict(obj)

    def predict_proba(self, obj):

        """ Predict the probability of hitting targets for a compound or compound DataFrame """

        if isinstance(obj, Mol):
            return self._m_predict_proba(obj)

        if isinstance(obj, pd.Series) or isinstance(obj, pd.DataFrame):
            return self._df_predict_proba(obj)

    def predict_log_proba(self, obj):

        """ Predict the log probability of hitting target for a compound or compound DataFrame """

        if isinstance(obj, Mol):
            return self._m_predict_log_proba(obj)

        if isinstance(obj, pd.Series) or isinstance(obj, pd.DataFrame):
            return self._df_predict_log_proba(obj)

    def _m_predict(self, m):

        """ Overload """

        raise NotImplementedError

    def _m_predict_proba(self, m):

        """ Overload """

        raise NotImplementedError

    def _m_predict_log_proba(self, m):

        """ Overload """

        raise NotImplementedError

    def _df_predict(self, df):

        """ Overload for prediction """

        raise NotImplementedError

    def _df_predict_proba(self, df):

        """ Overload for prediction """

        raise NotImplementedError

    def _df_predict_log_proba(self, df):

        """ Overload for prediction """

        raise NotImplementedError

    def __call__(self, obj):

        """ Predict the targets for a compound or compound DataFrame """

        self.predict(obj)



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD


"""
skchem.core

Module defining chemical types used in scikit-chem.

"""

from .base import ChemicalObject
from .point import Point3D
from .atom import Atom
from .bond import Bond
from .conformer import Conformer
from .mol import Mol



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.io.smiles

Defining input and output operations for smiles files.
"""

import skchem
import pandas as pd
from skchem.utils import Suppressor

def read_smiles(smiles_file, smiles_column=0, name_column=None, delimiter='\t', \
                        title_line=False, force=False, *args, **kwargs):

    """
    Read a smiles file into a pandas dataframe.  The class wraps the pandas read_csv function.

    @param smiles_file      A file path provided as a :str:, or a :file-like: object.
    @param smiles_column    The column index as an :int: in which the smiles strings are provided.
                            Defaults to _zero_.
    @param name_column      The column index as an :int: in which compound names are provided,
                            for use as the index in the dataframe.  Defaults to _None_.
    @param delimiter        The delimiter used, specified as a :str:.
                            Defaults to _<tab>_.
    @param title_line       A :bool: specifying whether a title line is provided,
                            to use as column titles.
    @param force            A :bool: specifying whether poorly parsed molecules should be skipped,
                            or an error thrown.
    Additionally, pandas read_csv arguments may be provided.

    @returns df             A dataframe of type :pandas.core.frame.DataFrame:.
    """

    with Suppressor():

        # set the header line to pass to the pandas parser
        # we accept True as being line zero, as is usual for smiles
        # if user specifies a header already, then do nothing

        header = kwargs.get('header', None)
        if title_line is True:
            header = 0
        elif header is not None:
            kwargs.pop('header') #remove from the kwargs to not pass it twice
        else:
            header = None

        # open file if not already open
        if isinstance(smiles_file, str):
            smiles_file = open(smiles_file, 'r')

        # read the smiles file
        df = pd.read_csv(smiles_file, delimiter=delimiter, header=header, *args, **kwargs)

        # replace the smiles column with the structure column
        lst = list(df.columns)
        lst[smiles_column] = 'structure'
        df.columns = lst

        # apply the from smiles constructor
        if force:
            def parse(smiles):

                """
                Parse a molecule from smiles string and return None if it doesn't load
                (restoring rdkit functionality)
                """

                try:
                    return skchem.Mol.from_smiles(smiles)
                except ValueError:
                    return None
        else:
            def parse(smiles):
                """
                Parse a molecule from smiles string
                """
                return skchem.Mol.from_smiles(smiles)

        df['structure'] = df['structure'].apply(str).apply(parse) #make sure is a string

        if force:
            df = df[df['structure'].notnull()]

        # set index if passed
        if name_column is not None:
            df = df.set_index(df.columns[name_column])

        return df

@classmethod
def from_smiles(_, *args, **kwargs):
    """ Create a DataFrame from a smiles file """
    return read_smiles(*args, **kwargs)

#set on pandas dataframe
pd.DataFrame.from_smiles = from_smiles



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

# The map functions are a stand in before parallelism is applied,
# so ignore the errors for using map + lambdas.

# pylint: disable=W0110

"""
skchem.target_prediction.PIDGIN

Wrapper for the PIDGIN models.
"""

import pandas as pd
from rdkit.Chem.rdMolDescriptors import GetMorganFingerprintAsBitVect
import gzip

# if cpickle available, import it.  otherwise use pickle
try:
    import cPickle as pickle
except ImportError:
    import pickle

from skchem.target_prediction import AbstractTargetPredictionAlgorithm
from skchem.descriptors import skchemize
from skchem.data import resource

class PIDGIN(AbstractTargetPredictionAlgorithm):

    """ Class implementing the PIDGIN target prediction algorithm """

    def __init__(self):
        with gzip.open(resource('PIDGIN_models.pkl.gz'), 'rb') as f:
            self.models = pickle.load(f)
        self.fingerprint = skchemize(GetMorganFingerprintAsBitVect, \
                                        radius=2, nBits=2048)
        self.targets = self.models.keys()

    def __call__(self, m):
        return self.predict_proba(m)

    def _m_predict(self, m):

        """ Predict binary binding profile for a molecule against 1080 protein targets """

        fp = self.fingerprint(m)
        return pd.Series((self.models[targ].predict(fp)[0] for targ in self.targets), \
                            index=self.targets)

    def _map_predict(self, m):

        """ Map based prediction for binary binding profile """

        fp = self.fingerprint(m)
        return pd.Series(map(lambda k: self.models[k].predict(fp), self.targets), \
                                                index=self.targets)

    def _m_predict_proba(self, m):

        """ Predict probability of molecule m binding to 1080 protein targets """

        fp = self.fingerprint(m)
        res = pd.Series(index=self.targets)
        for target, model in self.models.iteritems():
            res[target] = model.predict_proba(fp)[:, 1][0]
        return res

    def _map_predict_proba(self, m):

        """ Predict the log probability of molecule m binding to the 1080 proteins """

        fp = self.fingerprint(m)
        return pd.Series(map(lambda k: self.models[k].predict_proba(fp)[:, 1][0],\
                                                self.targets), index=self.targets)

    def _m_predict_log_proba(self, m):

        """ Predict the log probability of molecule m binding to the 1080 proteins """

        fp = self.fingerprint(m)
        res = pd.Series(index=self.targets)
        for target, model in self.models.iteritems():
            res[target] = model.predict_log_proba(fp)[:, 1][0]
        return res

    def map_predict_log_proba(self, m):

        """ Predict the log probabiltiy of molecule m binding to the 1080 proteins
        using map, for simple parallelism """

        fp = self.fingerprint(m)
        return pd.Series(map(lambda k: self.models[k].predict_log_proba(fp[:, 1][0]), \
                                                self.targets), index=self.targets)

    def _df_predict(self, df):

        """more efficient way to call the predict on large scikit-chem style dataframes"""

        fps = df.structure.apply(self.fingerprint)
        res = pd.DataFrame(index=fps.index, columns=self.targets)
        for target, model in self.models.iteritems():
            res[target] = model.predict(fps)
        return res

    def _df_map_predict(self, df):

        """ More efficient way to call the predict on large scikit-chem style dataframes,
        with a map implementation for easy parallelism"""

        fps = df.structure.apply(self.fingerprint)

        return pd.DataFrame(map(lambda k: self.models[k].predict(fps), self.targets), \
                                        columns=fps.index, index=self.targets).T


    def _df_predict_proba(self, df):

        """ More efficient way to call the predict_proba on large scikit-chem style dataframes"""

        fps = df.structure.apply(self.fingerprint)
        res = pd.DataFrame(index=fps.index, columns=self.targets)

        #parallelize here
        for target, model in self.models.iteritems():
            res[target] = model.predict_proba(fps)[:, 1]
        return res

    def _df_map_predict_proba(self, df):

        ''' map based way to call the predict_proba on large scikit-chem style dataframes'''

        fps = df.structure.apply(self.fingerprint)

        #parallize here trivially
        return pd.DataFrame(map(lambda k: self.models[k].predict_proba(fps)[:, 1], self.targets), \
                                            columns=fps.index, index=self.targets).T

    def _df_predict_log_proba(self, df):

        """ More efficient way to call the predict_proba on large scikit-chem style dataframes"""

        fps = df.structure.apply(self.fingerprint)
        res = pd.DataFrame(index=fps.index, columns=self.targets)

        for target, model in self.models.iteritems():
            res[target] = model.predict_log_proba(fps)[:, 1]
        return res

    def _df_map_predict_log_proba(self, df):

        """
        More efficient way to call the predict on large scikit-chem style dataframes,
        with a map implementation for easy parallelism
        """

        fps = df.structure.apply(self.fingerprint)

        return pd.DataFrame(map(lambda k: self.models[k].predict_log_proba(fps)[:, 1], \
                                        self.targets), columns=fps.index, index=self.targets).T



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.core.base

Define base classes for scikit chem objects
"""

class ChemicalObject(object):

    """ A mixin for each chemical object in scikit-chem """

    @classmethod
    def from_super(cls, obj):

        """A method that converts the class of an object of parent class to that of the child. """

        obj.__class__ = cls
        return obj
        


#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.core.conformer

Defining conformers in scikit-chem.
"""

import rdkit.Chem
from skchem.core import Point3D
from skchem.core import ChemicalObject

class Conformer(rdkit.Chem.rdchem.Conformer, ChemicalObject):

    """ Class representing a Conformer in scikit-chem. """
    #should use a view, list for now
    @property
    def atom_positions(self):

        """ Return the atom positions in the conformer for the atoms in the molecule. """

        return [Point3D.from_super(self.GetAtomPosition(i)) for i in range(self.GetNumAtoms())]

    @atom_positions.setter
    def atom_positions(self, value):

        """ Set the atom positions in the conformer.  Not implemented. """

        raise NotImplementedError

    @property
    def is_three_d(self):

        """ Return whether the conformer is three dimensional. """

        return self.is3D()

    @is_three_d.setter
    def is_three_d(self, value):

        """ Set whether the conformer is three dimensional. """

        self.set3D(value)

    def __repr__(self):
        return '<{klass} id="{id}" at {address}>'.format(klass=self.__class__.__name__, \
            id=self.GetId(), address=hex(id(self)))




#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

""" Tests for data functions """

from skchem.data import resource

def test_resource():

    """ Does resource target the the methane smiles test file?"""

    with open(resource('methane.smiles'), 'r') as f:
        assert f.readline() == 'C'



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.core.point

Defining points in scikit-chem.
"""

# the functionality here perhaps should be replaced by numpy arrays to get closer
# integration with the scientific python stack.

import rdkit.Geometry.rdGeometry
from skchem.core import ChemicalObject

class Point3D(rdkit.Geometry.rdGeometry.Point3D, ChemicalObject):

    """ Class representing a point in scikit-chem """

    def to_dict(self, two_d=True):

        """ Return a dictionary representation of the point """

        if two_d:
            return {"x": round(self.x), "y": round(self.y)}
        else:
            return {"x": round(self.x), "y": round(self.y), "z": round(self.z)}

    def __repr__(self):
        return '<{klass} coords="({x:.2f}, {y:.2f}, {z:.2f})" at {address}>'.format(
            klass=self.__class__.__name__, \
            x=self.x, \
            y=self.y, \
            z=self.z, \
            address=hex(id(self)))

    def __str__(self):
        return '({x:.2f}, {y:.2f}, {z:.2f})'.format(x=self.x, y=self.y, z=self.z)



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.io.sdf

Defining input and output operations for sdf files.
"""

from rdkit import Chem
import skchem
from skchem.utils import Suppressor
import pandas as pd

def read_sdf(sdf_file, force=False, *args, **kwargs):

    """
        Read an sdf file into a pandas dataframe.
        The function wraps the RDKit ForwardSDMolSupplier function.

        @param sdf_file     A file path provided as a :str:, or a :file-like: object.
        @param force        A :bool: specifying if the dataframe should be constructed even if
                            a molecule fails to parse correctly.  Defaults to _True_.

        Additionally, ForwardSDMolSupplier arguments may be passed.

        @returns df         A dataframe of type :pandas.core.frame.DataFrame:.

    """

    # use the suppression context manager to not pollute our stdout with rdkit errors and warnings.
    # perhaps this should be captured better by Mol etc.

    with Suppressor():

        if isinstance(sdf_file, str):
            sdf_file = open(sdf_file, 'rb') # use read bytes for python 3 compatibility

        # use a list as we need a resizing array as we don't know how many compounds there are.
        ms = []
        idx = []
        props = set()

        mol_supp = Chem.ForwardSDMolSupplier(sdf_file, *args, **kwargs)
        for i, m in enumerate(mol_supp):
            if m is None and force is False:
                # raise Value Error, like in the json module when no json is detected
                raise ValueError('Molecule {} could not be decoded.'.format(i + 1))
            elif m is None and force is True:
                continue
            ms.append(skchem.Mol(m))
            idx.append(m.GetProp('_Name'))
            props = props.union(props, set(m.GetPropNames()))

        df = pd.DataFrame(ms, index=idx, columns=['structure'])

        for prop in props:
            df[prop] = df.structure.apply(lambda m: m.props.get(prop, None))

        df.index.name = 'name'

        return df

@classmethod
def from_sdf(_, *args, **kwargs):

    """ Create a DataFrame from an sdf file """

    return read_sdf(*args, **kwargs)

#set on pandas dataframe
pd.DataFrame.from_sdf = from_sdf



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD


"""
skchem.core.atom

Defining atoms in scikit-chem.
"""

from rdkit import Chem
from skchem.core import ChemicalObject

class Atom(Chem.rdchem.Atom, ChemicalObject):

    """ Object representing an Atom in scikit-chem. """

    @property
    def element(self):

        """ Get the element of the atom as a string. """

        return self.GetSymbol()

    @element.setter
    def element(self, value):

        """ Set the element of the atom.  Not implemented. """

        raise NotImplementedError

    @property
    def props(self):

        """ Return a dictionary of properties of the atom. """

        return {i: self.GetProp() for i in self.GetProps()}

    @props.setter
    def props(self, value):

        """ Set the properties of the Atom.  Not implemented. """

        raise NotImplementedError

    def __repr__(self):

        return '<{klass} element="{element}" at {address}>'.format(
            klass=self.__class__.__name__,
            element=self.element,
            address=hex(id(self))
            )

    def __str__(self):
        return self.element



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.descriptors.fingerprints

Fingerprinting classes and associated functions are defined.

"""

import pandas as pd
from rdkit.Chem import DataStructs
import numpy as np
import skchem as skc

def skchemize(func, columns=None, *args, **kwargs):
    """

    transform an RDKit fingerprinting function to work well with pandas

    >>> from rdkit import Chem
    >>> import skchem
    >>> from skchem.descriptors import skchemize
    >>> from skchem.core import Mol
    >>> f = skchemize(Chem.RDKFingerprint)
    >>> m = Mol.from_smiles('c1ccccc1')
    >>> f(m)
    0       0
    1       0
    2       0
    3       0
    4       0
    5       0
    6       0
    7       0
    8       0
    9       0
    10      0
    11      0
    12      0
    13      0
    14      0
    15      0
    16      0
    17      0
    18      0
    19      0
    20      0
    21      0
    22      0
    23      0
    24      0
    25      0
    26      0
    27      0
    28      0
    29      0
           ..
    2018    0
    2019    0
    2020    0
    2021    0
    2022    0
    2023    0
    2024    0
    2025    0
    2026    0
    2027    0
    2028    0
    2029    0
    2030    0
    2031    0
    2032    0
    2033    0
    2034    0
    2035    0
    2036    0
    2037    0
    2038    0
    2039    0
    2040    0
    2041    0
    2042    0
    2043    0
    2044    0
    2045    0
    2046    0
    2047    0
    dtype: int64
    >>> from skchem.data import resource
    >>> df = skchem.read_sdf(resource('test_sdf', 'multi_molecule-simple.sdf'))
    >>> df.structure.apply(f)
          0     1     2     3     4     5     6     7     8     9     ...   2038  \\
    name                                                              ...          
    297      0     0     0     0     0     0     0     0     0     0  ...      0   
    6324     0     0     0     0     0     0     0     0     0     0  ...      0   
    6334     0     0     0     0     0     0     0     0     0     0  ...      0   
    <BLANKLINE>
          2039  2040  2041  2042  2043  2044  2045  2046  2047  
    name                                                        
    297      0     0     0     0     0     0     0     0     0  
    6324     0     0     0     0     0     0     0     0     0  
    6334     0     0     0     0     0     0     0     0     0  
    <BLANKLINE>
    [3 rows x 2048 columns]

    """

    def func_wrapper(m):

        """ Function that wraps an rdkit function allowing it to produce dataframes. """

        arr = np.array(0)
        DataStructs.ConvertToNumpyArray(func(m, *args, **kwargs), arr)

        return pd.Series(arr, index=columns)

    return func_wrapper

class Fingerprinter(object):

    """ Fingerprinter class. """

    def __init__(self, columns, func):
        self.columns = columns
        self.func = func

    @classmethod
    def from_rdkit_func(cls, func, columns=None, *args, **kwargs):

        """ Construct a Fingerprinter from an rdkit function. """

        return Fingerprinter(columns, func=skchemize(func, columns=columns, *args, **kwargs))

    def __call__(self, obj):
        return self.calculate(obj)

    def calculate(self, obj):

        """ calculate the fingerprint for the given object. """

        if isinstance(obj, skc.core.Mol):
            return self._calculate_m(obj)

        elif isinstance(obj, pd.DataFrame) or isinstance(obj, pd.Series):
            return self._calculate_df(obj)

        else:
            raise NotImplementedError

    def _calculate_m(self, m):

        """ calculate the fingerprint for a molecule. """

        return self.func(m)

    def _calculate_df(self, df):

        """ calculate a fingerprint for a dataframe of molecules """

        return df.structure.apply(self.func)



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.core.mol

Defining molecules in scikit-chem.
"""

import rdkit.Chem
from rdkit.Chem.rdDepictor import Compute2DCoords
from rdkit.Chem.rdMolDescriptors import CalcMolFormula

import json

from skchem.core import ChemicalObject, Atom, Bond, Conformer

class Mol(rdkit.Chem.rdchem.Mol, ChemicalObject):

    """
    Class representing a Molecule in scikit-chem.
    """

    def __init__(self, *args, **kwargs):
        super(Mol, self).__init__(*args, **kwargs)
        self.__two_d = None #set in constructor

    @property
    def name(self):

        """ Return the name of the molecule. """

        try:
            return self.GetProp('_Name')
        except KeyError:
            return None

    @name.setter
    def name(self, value):

        """ Set the name of the molecule. """

        if value is None:
            self.ClearProp('_Name')
        else:
            self.SetProp('_Name', value)

    @property
    def atoms(self):

        """ return an iterable over the atoms of the molecule. """

        return [Atom.from_super(self.GetAtomWithIdx(i)) for i in range(self.GetNumAtoms())]

    @atoms.setter
    def atoms(self, value):

        """ Set the atoms of a molecule.  Not implemented. """

        raise NotImplementedError

    @property
    def bonds(self):

        """ Return an iterable over the bonds of the molecule. """

        return [Bond.from_super(self.GetBondWithIdx(i)) for i in xrange(self.GetNumBonds())]

    @bonds.setter
    def bonds(self, value):

        """ Set the bonds of the molecule. Not implemented. """

        raise NotImplementedError

    # use a view to easily set properties?
    @property
    def props(self):

        """ Return a dictionary of the properties of the molecule. """

        return {i: self.GetProp(i) for i in self.GetPropNames()}

    @props.setter
    def props(self, value):

        """ Set the properties of the molecule. Not implemented. """

        raise NotImplementedError

    @property
    def conformers(self):

        """ Return a list of conformers of the molecule. """

        return [Conformer.from_super(self.GetConformer(i)) \
            for i in range(len(self.GetConformers()))]

    @conformers.setter
    def conformers(self, value):

        """ Set the conformers of the molecule.  Not implemented. """

        raise NotImplementedError

    def to_formula(self):

        """ Return the chemical formula of the molecule. """

        return CalcMolFormula(self)

    def _two_d(self):

        """ Return a conformer with coordinates in two dimension. """

        if not hasattr(self, '__two_d'):
            self.__two_d = Compute2DCoords(self)
        return self.conformers[self.__two_d]

    def to_dict(self, kind="chemdoodle"):

        """
        Return a dictionary representation of the molecule.

        TODO

        """

        if kind == "chemdoodle":
            return self._to_dict_chemdoodle()

        else:
            raise NotImplementedError

    def _to_dict_chemdoodle(self):

        """ Return a chemdoodle dict representation of the molecule. """

        atom_positions = [p.to_dict() for p in self._2D().atom_positions]
        atom_elements = [a.element for a in self.atoms]

        for i, atom_position in enumerate(atom_positions):
            atom_position['l'] = atom_elements[i]

        bonds = [b.to_dict() for b in self.bonds]

        return {"m": [{"a": atom_positions, "b": bonds}]}

    def to_json(self):

        """ Return a JSON representation of the molecule. """

        return json.dumps(self.to_dict())

    def to_inchi_key(self, *args, **kwargs):

        """ Return the INCHI key of the molecule. """

        return rdkit.Chem.InchiToInchiKey(self.to_inchi(), *args, **kwargs)

    def __repr__(self):
        return '<{klass} name="{name}" formula="{formula}" at {address}>'.format(
            klass=self.__class__.__name__,
            name=self.name,
            formula=self.to_formula(),
            address=hex(id(self)))

    def _repr_javascript(self):

        """ Rich printing in javascript. """

        return self.to_json()

    def __str__(self):
        return '<Mol: {}>'.format(self.to_smiles())

def bind_constructor(constructor_name, name_to_bind=None):

    """ Bind an (rdkit) constructor to the class """

    @classmethod
    def constructor(_, in_arg, name=None, *args, **kwargs):

        """ The constructor to be bound. """

        m = getattr(rdkit.Chem, 'MolFrom' + constructor_name)(in_arg, *args, **kwargs)
        if m is None:
            raise ValueError('Failed to parse molecule, {}'.format(in_arg))
        m = Mol.from_super(m)
        m.name = name
        return m

    setattr(Mol, 'from_{}'.format(constructor_name).lower() \
        if name_to_bind is None else name_to_bind, constructor)

def bind_serializer(serializer_name, name_to_bind=None):

    """ Bind an (rdkit) serializer to the class """

    def serializer(self, *args, **kwargs):

        """ The serializer to be bound. """
        return getattr(rdkit.Chem, 'MolTo' + serializer_name)(self, *args, **kwargs)

    setattr(Mol, 'to_{}'.format(serializer_name).lower() \
        if name_to_bind is None else name_to_bind, serializer)

CONSTRUCTORS = ['Inchi', 'Smiles', 'Mol2Block', 'Mol2File', 'MolBlock', \
                    'MolFile', 'PDBBlock', 'PDBFile', 'Smarts', 'TPLBlock', 'TPLFile']
SERIALIZERS = ['Inchi', 'Smiles', 'MolBlock', 'PDBBlock', 'Smarts', 'TPLBlock', 'TPLFile']

list(map(bind_constructor, CONSTRUCTORS))
list(map(bind_serializer, SERIALIZERS))




#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.io

Module defining input and output methods in scikit-chem.

"""

from .sdf import read_sdf
from .smiles import read_smiles



#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

from setuptools import *

DISTNAME = 'scikit-chem'
DESCRIPTION = 'A set of python modules for cheminformatics'
with open('README.md') as f:
    LONG_DESCRIPTION = f.read()
MAINTAINER = 'Richard Lewis'
MAINTAINER_EMAIL = 'rl403@cam.ac.uk'
URL = 'http://github.com/richlewis42/scikit-chem'
LICENSE = 'new BSD'
DOWNLOAD_URL = 'https://github.com/richlewis42/scikit-chem/archive/master.zip'
CLASSIFIERS = [ 
    'Development Status :: 1 - Planning',
    'Intended Audience :: Science/Research',
    'Intended Audience :: Education',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Natural Language :: English',
    'Programming Language :: Python',
    'Programming Language :: Python :: 2',
    'Programming Language :: Python :: 2.6',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Operating System :: Microsoft :: Windows',
    'Operating System :: POSIX',
    'Operating System :: Unix',
    'Operating System :: MacOS',
    'Topic :: Software Development',
    'Topic :: Scientific/Engineering :: Chemistry'
]
MAJOR = 0
MINOR = 0
MICRO = 1
VERSION = '{major}.{minor}.{micro}'.format(major=MAJOR, minor=MINOR, micro=MICRO)

def setup_package():

    metadata = dict(name=DISTNAME,
                    maintainer=MAINTAINER,
                    maintainer_email=MAINTAINER_EMAIL,
                    description=DESCRIPTION,
                    license=LICENSE,
                    url=URL,
                    version=VERSION,
                    download_url=DOWNLOAD_URL,
                    long_description=LONG_DESCRIPTION,
                    classifiers=CLASSIFIERS
    )

    setup(
        packages=find_packages(),
        package_data = {'skchem.target_prediction': ['data/PIDGIN_models.pkl.gz']},
        install_requires=[
            #'rdkit',  ## currently rdkit does not have a valid pip installable package, so this dependency must be met by the installer
            'pandas',
            'numpy',
            'scikit-learn'
            ],
        test_suite='nose.collector',
        tests_require=[
            'nose',
            'pytest'
            ],
        zip_safe=False,
        **metadata
        )


if __name__ == "__main__":
    setup_package()


#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.target_prediction

Module implementing target prediction algorithms.

"""

from .target_prediction import AbstractTargetPredictionAlgorithm
from .PIDGIN import PIDGIN
